@@ -56,7 +56,7 @@
 #else
 # include "compat/stdbool.h"
 #endif
-#if TIME_WITH_SYS_TIME
+#ifdef TIME_WITH_SYS_TIME
 # include <time.h>
 #endif
 #ifndef HAVE_STRUCT_TIMESPEC
@@ -115,7 +115,7 @@ lock_file(int fd, int lockit)
     }
     debug_return_bool(lockf(fd, op, 0) == 0);
 }
-#elif HAVE_FLOCK
+#elif defined(HAVE_FLOCK)
 bool
 lock_file(int fd, int lockit)
 {
@@ -168,7 +168,8 @@ lock_file(int fd, int lockit)
 ssize_t
 sudo_parseln(char **bufp, size_t *bufsizep, unsigned int *lineno, FILE *fp)
 {
-    size_t len, linesize = 0, total = 0;
+    size_t linesize = 0, total = 0;
+    ssize_t len;
     char *cp, *line = NULL;
     bool continued;
     debug_decl(sudo_parseln, SUDO_DEBUG_UTIL)
