@@ -1,5 +1,6 @@
 /*
- * Copyright (c) 2008, 2010 Todd C. Miller <Todd.Miller@courtesan.com>
+ * Copyright (c) 2008, 2010-2011, 2013
+ *	Todd C. Miller <Todd.Miller@courtesan.com>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -16,6 +17,8 @@
 
 #include <config.h>
 
+#ifndef HAVE_ISBLANK
+
 #include <sys/types.h>
 
 #include "missing.h"
@@ -26,3 +29,4 @@ isblank(int ch)
 {
     return ch == ' ' || ch == '\t';
 }
+#endif /* HAVE_ISBLANK */
