@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2010-2011 Todd C. Miller <Todd.Miller@courtesan.com>
+ * Copyright (c) 2010-2012 Todd C. Miller <Todd.Miller@courtesan.com>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -32,10 +32,12 @@
 
 #include "missing.h"
 
+__dso_public int main(int argc, char *argv[]);
+
 int
 main(int argc, char *argv[])
 {
-    static char *my_sys_siglist[NSIG];
+    static char *sudo_sys_siglist[NSIG];
     int i;
 
 #include "compat/mksiglist.h"
@@ -43,10 +45,10 @@ main(int argc, char *argv[])
     printf("#include <config.h>\n");
     printf("#include <signal.h>\n");
     printf("#include \"missing.h\"\n\n");
-    printf("const char *const my_sys_siglist[NSIG] = {\n");
+    printf("const char *const sudo_sys_siglist[NSIG] = {\n");
     for (i = 0; i < NSIG; i++) {
-	if (my_sys_siglist[i] != NULL) {
-	    printf("    \"%s\",\n", my_sys_siglist[i]);
+	if (sudo_sys_siglist[i] != NULL) {
+	    printf("    \"%s\",\n", sudo_sys_siglist[i]);
 	} else {
 	    printf("    \"Signal %d\",\n", i);
 	}
