@@ -198,6 +198,10 @@ struct sudo_defs_types sudo_defs_table[] = {
 	"badpass_message", T_STR,
 	N_("Incorrect password message: %s"),
 	NULL,
+    }, {
+	"lecture_status_dir", T_STR|T_PATH,
+	N_("Path to lecture status dir: %s"),
+	NULL,
     }, {
 	"timestampdir", T_STR|T_PATH,
 	N_("Path to authentication timestamp dir: %s"),
@@ -374,6 +378,10 @@ struct sudo_defs_types sudo_defs_table[] = {
 	"maxseq", T_UINT,
 	N_("Maximum I/O log sequence number: %u"),
 	NULL,
+    }, {
+	"use_netgroups", T_FLAG,
+	N_("Enable sudoers netgroup support"),
+	NULL,
     }, {
 	NULL, 0, NULL
     }
