@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999-2005, 2007-2013
+ * Copyright (c) 1999-2005, 2007-2014
  *	Todd C. Miller <Todd.Miller@courtesan.com>
  *
  * Permission to use, copy, modify, and distribute this software for any
@@ -435,6 +435,9 @@ init_defaults(void)
 #else
     def_pam_session = true;
 #endif
+#ifdef HAVE_INNETGR
+    def_use_netgroups = true;
+#endif
 
     /* Syslog options need special care since they both strings and ints */
 #if (LOGGING & SLOG_SYSLOG)
@@ -467,6 +470,7 @@ init_defaults(void)
     def_mailto = estrdup(MAILTO);
     def_mailsub = estrdup(N_(MAILSUBJECT));
     def_badpass_message = estrdup(_(INCORRECT_PASSWORD));
+    def_lecture_status_dir = estrdup(_PATH_SUDO_LECTURE_DIR);
     def_timestampdir = estrdup(_PATH_SUDO_TIMEDIR);
     def_passprompt = estrdup(_(PASSPROMPT));
     def_runas_default = estrdup(RUNAS_DEFAULT);
