@@ -38,11 +38,13 @@
 # include "compat/stdbool.h"
 #endif /* HAVE_STDBOOL_H */
 
+#define DEFAULT_TEXT_DOMAIN	"sudoers"
+#include "gettext.h"		/* must be included before missing.h */
+
 #include "missing.h"
 #include "fatal.h"
 #include "alloc.h"
 #include "logging.h"
-#include "gettext.h"
 
 static int current_locale = SUDOERS_LOCALE_USER;
 static char *user_locale;
