@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1993-1996, 1998-2005, 2007-2013
+ * Copyright (c) 1993-1996, 1998-2005, 2007-2014
  *	Todd C. Miller <Todd.Miller@courtesan.com>
  *
  * Permission to use, copy, modify, and distribute this software for any
@@ -29,6 +29,9 @@
 # include "compat/stdbool.h"
 #endif /* HAVE_STDBOOL_H */
 
+#define DEFAULT_TEXT_DOMAIN	"sudoers"
+#include "gettext.h"		/* must be included before missing.h */
+
 #include <pathnames.h>
 #include "missing.h"
 #include "fatal.h"
@@ -42,9 +45,6 @@
 #include "sudo_debug.h"
 #include "sudo_util.h"
 
-#define DEFAULT_TEXT_DOMAIN	"sudoers"
-#include "gettext.h"
-
 /*
  * Password db and supplementary group IDs with associated group names.
  */
@@ -236,7 +236,7 @@ int check_user(int validate, int mode);
 bool user_is_exempt(void);
 
 /* prompt.c */
-char *expand_prompt(const char *old_prompt, const char *user, const char *host);
+char *expand_prompt(const char *old_prompt, const char *auth_user);
 
 /* timestamp.c */
 void remove_timestamp(bool);
@@ -326,7 +326,7 @@ void sudo_setspent(void);
 char *get_timestr(time_t, int);
 
 /* boottime.c */
-int get_boottime(struct timeval *);
+bool get_boottime(struct timeval *);
 
 /* iolog.c */
 int io_set_max_sessid(const char *sessid);
